@@ -1,5 +1,6 @@
+# MIT License
 #
-# Copyright (c) 2001 - 2019 The SCons Foundation
+# Copyright The SCons Foundation
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -19,33 +20,33 @@
 # LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 # OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-#
 
-__revision__ = "/src/script/src/engine/SCons/Tool/MSCommon/vs.py none 2019-12-17 02:06:27 none"
-
-__doc__ = """Module to detect Visual Studio and/or Visual C/C++
+"""
+MS Compilers: detect Visual Studio and/or Visual C/C++
 """
 
 import os
 
 import SCons.Errors
+import SCons.Tool.MSCommon.vc
 import SCons.Util
 
-from .common import debug, \
-                   get_output, \
-                   is_win64, \
-                   normalize_env, \
-                   parse_output, \
-                   read_reg
+from .common import (
+    debug,
+    get_output,
+    is_win64,
+    normalize_env,
+    parse_output,
+    read_reg,
+)
 
-import SCons.Tool.MSCommon.vc
 
-class VisualStudio(object):
+class VisualStudio:
     """
     An abstract base class for trying to find installed versions of
     Visual Studio.
     """
-    def __init__(self, version, **kw):
+    def __init__(self, version, **kw) -> None:
         self.version = version
         kw['vc_version']  = kw.get('vc_version', version)
         kw['sdk_version'] = kw.get('sdk_version', version)
@@ -55,61 +56,60 @@ class VisualStudio(object):
     def find_batch_file(self):
         vs_dir = self.get_vs_dir()
         if not vs_dir:
-            debug('find_executable():  no vs_dir')
+            debug('no vs_dir')
             return None
         batch_file = os.path.join(vs_dir, self.batch_file_path)
         batch_file = os.path.normpath(batch_file)
         if not os.path.isfile(batch_file):
-            debug('find_batch_file():  %s not on file system' % batch_file)
+            debug('%s not on file system', batch_file)
             return None
         return batch_file
 
-    def find_vs_dir_by_vc(self):
-        SCons.Tool.MSCommon.vc.get_installed_vcs()
-        dir = SCons.Tool.MSCommon.vc.find_vc_pdir(self.vc_version)
+    def find_vs_dir_by_vc(self, env):
+        dir = SCons.Tool.MSCommon.vc.find_vc_pdir(env, self.vc_version)
         if not dir:
-            debug('find_vs_dir_by_vc():  no installed VC %s' % self.vc_version)
+            debug('no installed VC %s', self.vc_version)
             return None
         return os.path.abspath(os.path.join(dir, os.pardir))
 
-    def find_vs_dir_by_reg(self):
+    def find_vs_dir_by_reg(self, env):
         root = 'Software\\'
 
         if is_win64():
             root = root + 'Wow6432Node\\'
         for key in self.hkeys:
             if key=='use_dir':
-                return self.find_vs_dir_by_vc()
+                return self.find_vs_dir_by_vc(env)
             key = root + key
             try:
                 comps = read_reg(key)
-            except SCons.Util.WinError as e:
-                debug('find_vs_dir_by_reg(): no VS registry key {}'.format(repr(key)))
+            except OSError:
+                debug('no VS registry key %s', repr(key))
             else:
-                debug('find_vs_dir_by_reg(): found VS in registry: {}'.format(comps))
+                debug('found VS in registry: %s', comps)
                 return comps
         return None
 
-    def find_vs_dir(self):
+    def find_vs_dir(self, env):
         """ Can use registry or location of VC to find vs dir
         First try to find by registry, and if that fails find via VC dir
         """
 
-        vs_dir=self.find_vs_dir_by_reg()
+        vs_dir = self.find_vs_dir_by_reg(env)
         if not vs_dir:
-            vs_dir = self.find_vs_dir_by_vc()
-        debug('find_vs_dir(): found VS in ' + str(vs_dir ))
+            vs_dir = self.find_vs_dir_by_vc(env)
+        debug('found VS in %s', str(vs_dir))
         return vs_dir
 
-    def find_executable(self):
-        vs_dir = self.get_vs_dir()
+    def find_executable(self, env):
+        vs_dir = self.get_vs_dir(env)
         if not vs_dir:
-            debug('find_executable():  no vs_dir ({})'.format(vs_dir))
+            debug('no vs_dir (%s)', vs_dir)
             return None
         executable = os.path.join(vs_dir, self.executable_path)
         executable = os.path.normpath(executable)
         if not os.path.isfile(executable):
-            debug('find_executable():  {} not on file system'.format(executable))
+            debug('%s not on file system', executable)
             return None
         return executable
 
@@ -121,21 +121,21 @@ class VisualStudio(object):
             self._cache['batch_file'] = batch_file
             return batch_file
 
-    def get_executable(self):
+    def get_executable(self, env=None):
         try:
-            debug('get_executable using cache:%s'%self._cache['executable'])
+            debug('using cache:%s', self._cache['executable'])
             return self._cache['executable']
         except KeyError:
-            executable = self.find_executable()
+            executable = self.find_executable(env)
             self._cache['executable'] = executable
-            debug('get_executable not in cache:%s'%executable)
+            debug('not in cache:%s', executable)
             return executable
 
-    def get_vs_dir(self):
+    def get_vs_dir(self, env=None):
         try:
             return self._cache['vs_dir']
         except KeyError:
-            vs_dir = self.find_vs_dir()
+            vs_dir = self.find_vs_dir(env)
             self._cache['vs_dir'] = vs_dir
             return vs_dir
 
@@ -148,7 +148,7 @@ class VisualStudio(object):
             self._cache['supported_arch'] = self.supported_arch
             return self.supported_arch
 
-    def reset(self):
+    def reset(self) -> None:
         self._cache = {}
 
 # The list of supported Visual Studio versions we know how to detect.
@@ -198,6 +198,18 @@ class VisualStudio(object):
 # Tool/MSCommon/vc.py, and the MSVC_VERSION documentation in Tool/msvc.xml.
 
 SupportedVSList = [
+    # Visual Studio 2022
+    VisualStudio('14.3',
+                 vc_version='14.3',
+                 sdk_version='10.0A',
+                 hkeys=[],
+                 common_tools_var='VS170COMNTOOLS',
+                 executable_path=r'Common7\IDE\devenv.com',
+                 # should be a fallback, prefer use vswhere installationPath
+                 batch_file_path=r'Common7\Tools\VsDevCmd.bat',
+                 supported_arch=['x86', 'amd64', "arm", 'arm64'],
+                 ),
+
     # Visual Studio 2019
     VisualStudio('14.2',
                  vc_version='14.2',
@@ -205,8 +217,9 @@ SupportedVSList = [
                  hkeys=[],
                  common_tools_var='VS160COMNTOOLS',
                  executable_path=r'Common7\IDE\devenv.com',
-                 batch_file_path=r'VC\Auxiliary\Build\vsvars32.bat',
-                 supported_arch=['x86', 'amd64', "arm"],
+                 # should be a fallback, prefer use vswhere installationPath
+                 batch_file_path=r'Common7\Tools\VsDevCmd.bat',
+                 supported_arch=['x86', 'amd64', "arm", 'arm64'],
                  ),
 
     # Visual Studio 2017
@@ -216,10 +229,23 @@ SupportedVSList = [
                  hkeys=[],
                  common_tools_var='VS150COMNTOOLS',
                  executable_path=r'Common7\IDE\devenv.com',
-                 batch_file_path=r'VC\Auxiliary\Build\vsvars32.bat',
-                 supported_arch=['x86', 'amd64', "arm"],
+                 # should be a fallback, prefer use vswhere installationPath
+                 batch_file_path=r'Common7\Tools\VsDevCmd.bat',
+                 supported_arch=['x86', 'amd64', "arm", 'arm64'],
                  ),
 
+    # Visual C++ 2017 Express Edition (for Desktop)
+    VisualStudio('14.1Exp',
+                 vc_version='14.1',
+                 sdk_version='10.0A',
+                 hkeys=[],
+                 common_tools_var='VS150COMNTOOLS',
+                 executable_path=r'Common7\IDE\WDExpress.exe',
+                 # should be a fallback, prefer use vswhere installationPath
+                 batch_file_path=r'Common7\Tools\VsDevCmd.bat',
+                 supported_arch=['x86', 'amd64', "arm", 'arm64'],
+    ),
+
     # Visual Studio 2015
     VisualStudio('14.0',
                  vc_version='14.0',
@@ -365,8 +391,8 @@ SupportedVSList = [
     VisualStudio('7.0',
                  sdk_version='2003R2',
                  hkeys=[r'Microsoft\VisualStudio\7.0\Setup\VS\ProductDir'],
-                 common_tools_var='VS70COMNTOOLS',
-                 executable_path=r'IDE\devenv.com',
+                 common_tools_var='VSCOMNTOOLS',
+                 executable_path=r'Common7\IDE\devenv.com',
                  batch_file_path=r'Common7\Tools\vsvars32.bat',
                  default_dirname='Microsoft Visual Studio .NET',
                  supported_arch=['x86'],
@@ -377,7 +403,7 @@ SupportedVSList = [
                  sdk_version='2003R1',
                  hkeys=[r'Microsoft\VisualStudio\6.0\Setup\Microsoft Visual Studio\ProductDir',
                         'use_dir'],
-                 common_tools_var='VS60COMNTOOLS',
+                 common_tools_var='MSDevDir',
                  executable_path=r'Common\MSDev98\Bin\MSDEV.COM',
                  batch_file_path=r'Common7\Tools\vsvars32.bat',
                  default_dirname='Microsoft Visual Studio',
@@ -399,21 +425,21 @@ for vs in SupportedVSList:
 InstalledVSList = None
 InstalledVSMap  = None
 
-def get_installed_visual_studios():
+def get_installed_visual_studios(env=None):
     global InstalledVSList
     global InstalledVSMap
     if InstalledVSList is None:
         InstalledVSList = []
         InstalledVSMap = {}
         for vs in SupportedVSList:
-            debug('trying to find VS %s' % vs.version)
-            if vs.get_executable():
-                debug('found VS %s' % vs.version)
+            debug('trying to find VS %s', vs.version)
+            if vs.get_executable(env):
+                debug('found VS %s', vs.version)
                 InstalledVSList.append(vs)
                 InstalledVSMap[vs.version] = vs
     return InstalledVSList
 
-def reset_installed_visual_studios():
+def reset_installed_visual_studios() -> None:
     global InstalledVSList
     global InstalledVSMap
     InstalledVSList = None
@@ -458,21 +484,21 @@ def reset_installed_visual_studios():
 #    for variable, directory in env_tuple_list:
 #        env.PrependENVPath(variable, directory)
 
-def msvs_exists():
-    return (len(get_installed_visual_studios()) > 0)
+def msvs_exists(env=None) -> bool:
+    return len(get_installed_visual_studios(env)) > 0
 
 def get_vs_by_version(msvs):
     global InstalledVSMap
     global SupportedVSMap
 
-    debug('get_vs_by_version()')
+    debug('called')
     if msvs not in SupportedVSMap:
         msg = "Visual Studio version %s is not supported" % repr(msvs)
         raise SCons.Errors.UserError(msg)
     get_installed_visual_studios()
     vs = InstalledVSMap.get(msvs)
-    debug('InstalledVSMap:%s'%InstalledVSMap)
-    debug('get_vs_by_version: found vs:%s'%vs)
+    debug('InstalledVSMap:%s', InstalledVSMap)
+    debug('found vs:%s', vs)
     # Some check like this would let us provide a useful error message
     # if they try to set a Visual Studio version that's not installed.
     # However, we also want to be able to run tests (like the unit
@@ -507,8 +533,9 @@ def get_default_version(env):
         if versions:
             env['MSVS_VERSION'] = versions[0] #use highest version by default
         else:
-            debug('get_default_version: WARNING: no installed versions found, '
-                  'using first in SupportedVSList (%s)'%SupportedVSList[0].version)
+            debug('WARNING: no installed versions found, '
+                  'using first in SupportedVSList (%s)',
+                  SupportedVSList[0].version)
             env['MSVS_VERSION'] = SupportedVSList[0].version
 
     env['MSVS']['VERSION'] = env['MSVS_VERSION']
@@ -537,15 +564,16 @@ def get_default_arch(env):
 
     return arch
 
-def merge_default_version(env):
+def merge_default_version(env) -> None:
     version = get_default_version(env)
     arch = get_default_arch(env)
 
-def msvs_setup_env(env):
-    batfilename = msvs.get_batch_file()
+# TODO: refers to versions and arch which aren't defined; called nowhere. Drop?
+def msvs_setup_env(env) -> None:
     msvs = get_vs_by_version(version)
     if msvs is None:
         return
+    batfilename = msvs.get_batch_file()
 
     # XXX: I think this is broken. This will silently set a bogus tool instead
     # of failing, but there is no other way with the current scons tool
